###########################################################################################################
#### Script: MSc thesis Forecasting Drought Impacts
#### Date: 2019-03
#### Auther: Melati van der Weert
#### Supervisors: Samuel Sutanto, Henny van Lanen, and Niko Wanders
#### Chair group: Hydrology and Quantitative Water Management Group
#### University: Wageningen University & Research, the Netherlands
#### Objective: Create drought impact functions based on random forest using a subset of the 
#### observational dataset to train the model
#### Instructions: Go through the script step by step and execute the R-code (select code and Ctrl-R).
#### If required set the working folder, input folder, and output folder according to your specifications
###########################################################################################################


## Load required packages
library(readr)
library(psych) 
library(car) 
library(plyr) 
library("MASS")
library(mgcv)
library(gam)
library(readr)
library(DescTools)
library(randomForest)
library(caret)

## number and names of the NUTS1 regions in Germany
list_NUTS = c("741","743","750","753","775","776","777","791","796","800","810","835","843","858","903","979")

## go through every NUTS 1 region
for (NUTS in list_NUTS){ print(NUTS )
  
  ## go trough every impact category
  for (category in 1:4){
  
    ## load model input
    model_input_data = read_csv(paste0("F:/Thesis/Data/3_Random_Forest/Historical_Analysis/Case_4/Model_Input/Binary/",NUTS,"/Impact_Combined_",category,".csv"),
                                col_types = cols(X1 = col_skip()))
    
    ## only built drought impact functions if it has sufficient drought impact occuramce data
    if(sum(model_input_data$Impact)>10){
    
      ## define training and test set
      train_data <- model_input_data[1:288,]
      train_data$test=NULL
      test_data <- model_input_data[289:324,]
      test_data$test=NULL
      test_data$Impact =NULL
      
      ## only built drought impact functions if it has sufficient drought impact occuramce data in the training dataset
      if(sum(train_data$Impact)>0){
      
        ## create dataframe for timeseries
        tmp_data_frame = data.frame("Year" = test_data$Year ,"Month" = test_data$Month,
                                    "Impact_Probability" = 0)
        
        ## define formula
        formula = as.formula(as.factor(Impact)~SPI_1+SPI_3+SPI_6+SPI_12+SPEI_1+SPEI_3+SPEI_6+SPEI_12+SRI_1+SRI_3+SRI_6+SRI_12+Month+Year)
        
        ## built model
        best_model=randomForest(formula,data=train_data,importance = TRUE,ntree=1000)
        
        ## save model
        saveRDS(best_model,paste0("F:/Thesis/Data/3_Random_Forest/Historical_Analysis/Case_4/Model/",NUTS,"/Impact_Combined_",category,".rds"))
        
        ## predict drought impact for selected year
        tmp_data_frame$Impact_Probability = predict(best_model,newdata = test_data, type="prob")[,2]
        
        ## write predicted drought impact for the whole period in a CSV file
        write.csv(tmp_data_frame,paste0('F:/Thesis/Data/3_Random_Forest/Historical_Analysis/Case_4/Modelled_Drought_Impact/',NUTS,"/Impact_Combined_",category,".csv"))
        
        
        ## save plot
        jpeg(paste('F:/Thesis/Figures/3_Random_Forest/Historical_Analysis/Case_4/Modelled_Drought_Impact/',NUTS,"_Impact_Combined_",category,".jpg", sep="")) 
        
        ## plot predicted timeserie versus observed EDII reports
        plot(seq(as.Date("2015-01-01"), as.Date("2017-12-01"), "months"),(model_input_data$Impact[289:324]),t="l",xlab = "Date", ylab = "Impact", col = 'dark gray',main = paste("Total Impact in", "Germany" ,sep=" "))
        lines(seq(as.Date("2015-01-01"), as.Date("2017-12-01"), "months"),tmp_data_frame$Impact_Probability,col = 'red')
        
        ## close device
        dev.off()
      }
    }  
  }
}











